package org.openscm.kundo.plugins

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.openscm.kundo.plugins.context.BuildContext
import java.util.jar.JarOutputStream

 /**
 * Ear Delegate
 * @author <a href="mailto:ramesh.ramakrishnan@upco.co.uk">Ramesh Ramakrishnan</a>
 * @version 1.0.0
 * 
 * Description: A  Delegate for the creation of project EAR files
 * 
 * <p></p>
 */
/* 
 * @ant-target doEar
 */
class EarDelegate extends AbstractPluginTargetDelegate {
	
	private static final EAR_SCOPES = ["runtime"]
	private String ps = File.pathSeparator
	/*
	 * Map to EarContent folder location of the EnterpriseApplication project
	 * @ant-property-name ear.enterprise.folder
	 */
	protected String enterpriseFolder
	/*
	 * Value to determine if compression is needed while archiving. 
	 * @ant-property-name ear.compress
	 */
	protected String compress
	/*
	 * For entries coming from existing archives, keep compression as it has been originally instead of compress attribute. 
	 * @ant-property-name ear.keep.compression
	 */
	protected String keepCompression
	/*
	 * Character encoding to use for filenames inside the archive. 
	 * @ant-property-name ear.encoding
	 */
	protected String encoding
	/*
	 * Indicates to update or override destination file if exists already. 
	 * @ant-property-name ear.update
	 */
	protected String update
	/*
	 * Behavior when a duplicate file is found. 
	 * @ant-property-name ear.duplicate
	 */
	protected String duplicate
	/*
	 * Whether file modification time will be rounded up to next even number of seconds. 
	 * @ant-property-name ear.roundup
	 */
	protected String roundup
	/*
	 * Non default level at which file compression should be performed.
	 * @ant-property-name ear.level
	 */
	protected String level
	/*
	 * Manifest file to use
	 * @ant-property-name ear.manifest
	 */
	protected String manifest
	/*
	 * Destination folder where the EAR archive to be placed
	 * @ant-property-name ear.destination.folder
	 */
	protected String destinationFolder
	/*
	 * Behaviour when no files match.
	 * @ant-property-name ear.when.manifest.only
	 */
	protected String whenManifestOnly
	/*
	 * Behavior when a manifest is found in zipfileset or zipgroupfileset.  
	 * @ant-property-name ear.fileset.manifest
	 */
	protected String filesetManifest
	/*
	 * Encoding used to read jar manifest. when not specified, defaults to platform encoding 
	 * @ant-property-name ear.manifest.encoding
	 */
	protected String manifestEncoding
	/*
	 * Folder where the runtime libraries are copied in to 
	 * @ant-property-name ear.libraries.folder
	 */
	protected String librariesFolder
	/*
	 * Archive type.
	 * @ant-property-name ear.type
	 */
	protected String type

	
	/**
	 * Constructor sets ant and buildContext instances in super class
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	 EarDelegate( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext)
	}
	
	/**
	 * Setter method for member variable enterpriseFolder
	 * @param enterpriseFolder	
	 */	
	void setEnterpriseFolder( String enterpriseFolder ){
		this.enterpriseFolder = enterpriseFolder
		if( !( enterpriseFolder.endsWith( '/' ) || ( enterpriseFolder.endsWith( '\\' ) ) ) )
			this.enterpriseFolder = this.enterpriseFolder + '/'
	}
	/**
	 * Setter method for member variable compress
	 * @param compress	
	 */	
	void setCompress( String compress ){
		this.compress = compress
	}
	/**
	 * Setter method for member variable keepCompression
	 * @param keepCompression	
	 */	
	void setKeepCompression( String keepCompression ){
		this.keepCompression = keepCompression
	}
	/**
	 * Setter method for member variable encoding
	 * @param encoding	
	 */	
	void setEncoding( String encoding ){
		this.encoding = encoding
	}
	/**
	 * Setter method for member variable update
	 * @param update	
	 */	
	void setUpdate( String update ){
		this.update = update
	}
	/**
	 * Setter method for member variable duplicate
	 * @param duplicate	
	 */	
	void setDuplicate( String duplicate ){
		this.duplicate = duplicate
	}
	/**
	 * Setter method for member variable roundup
	 * @param roundup	
	 */	
	void setRoundup( String roundup ){
		this.roundup = roundup
	}
	/**
	 * Setter method for member variable level
	 * @param level	
	 */	
	void setLevel( String level ){
		this.level = level
	}
	/**
	 * Setter method for member variable manifest
	 * @param manifest	
	 */	
	void setManifest( String manifest ){
		this.manifest = manifest
	}
	/**
	 * Setter method for member variable whenManifestOnly
	 * @param whenManifestOnly	
	 */	
	void setWhenManifestOnly( String whenManifestOnly ){
		this.whenManifestOnly = whenManifestOnly
	}
	/**
	 * Setter method for member variable filesetManifest
	 * @param filesetManifest	
	 */	
	void setFilesetManifest( String filesetManifest ){
		this.filesetManifest = filesetManifest
	}
	/**
	 * Setter method for member variable manifestEncoding
	 * @param manifestEncoding	
	 */	
	void setManifestEncoding( String manifestEncoding ){
		this.manifestEncoding = manifestEncoding
	}
	/**
	 * Setter method for member variable destinationFolder
	 * @param destinationFolder	
	 */	
	void setDestinationFolder( String destinationFolder ){
		this.destinationFolder = destinationFolder
		if( !( destinationFolder.endsWith( '/' ) || ( destinationFolder.endsWith( '\\' ) ) ) )
			this.destinationFolder = this.destinationFolder + '/'
	}
	/**
	 * Setter method for member variable librariesFolder
	 * @param librariesFolder	
	 */	
	void setLibrariesFolder( String librariesFolder ){
		this.librariesFolder = librariesFolder
	}
	/**
	 * Setter method for member variable type
	 * @param type	
	 */	
	void setType( String type ){
		this.type = type
	}


	
	/**
	 * Implementation of abstract method from AbstractPluginTargetDelegate
	 * to hold assertions that all global and local properties are available to the plugin.
	 * This method is automatically called prior to execution.
	 */	
	void doCheck(){
		assert enterpriseFolder != null
		assert compress != null
		assert keepCompression != null
		assert encoding != null
		assert update != null
		assert duplicate != null
		assert roundup != null
		assert level != null
		assert manifest != null
		assert destinationFolder != null
		assert librariesFolder != null
		assert type != null
		assert buildContext.get( "project.properties" ) != null

	}	
	
	/**
	 * Concrete implementation of the AbstractDelegate.execute() method
	 */
	void doExecute(){		
		// Retrieve the indexed project properties from the global context
		Map properties = buildContext.get( "project.properties" )
		String artifactName = ant.project.getProperty( "global.artifactid" )
		String earDest = destinationFolder + artifactName + '.' + type
		String appXml = enterpriseFolder + 'META-INF/application.xml'
		def earOptions = [destfile:earDest, 
		                  compress:compress,
		                  keepcompression:keepCompression,
		                  encoding:encoding,
		                  update:update,
		                  duplicate:duplicate,
		                  roundup:roundup,
		                  filesetmanifest:filesetManifest,
		                  whenmanifestonly:whenManifestOnly
		                 ]
		// check if a manifest file has been included, if not we want a default manifest building for us by ant
		if ( !manifest.equals( "" ) )
			earOptions.put( "manifest", manifest )
		if( !manifestEncoding.equals(""))
			earOptions.put( "manifestencoding", manifestEncoding )
		if( !level.equals(""))
			earOptions.put( "level", level )
		if( update.equals("false")) 
			earOptions.put( "appxml", appXml )
		

		EAR_SCOPES.each{ scope ->
			if(buildContext.isSet("${artifactName}.${scope}.classpath" )){
				String dependencies = buildContext.get( "${artifactName}.${scope}.classpath" )		
				// Delete the folder before performing a fresh copy
				ant.delete(dir:librariesFolder)
				dependencies.split( "${ps}" ).each{	dependency ->					
					def libString = dependency.toString()
					ant.copy( file:dependency, todir:librariesFolder) 
				}
			}
		}
		
		// Rename runtime libraries so as to make sure the version is stripped off
		renameFiles(librariesFolder)
		
		ant.ear( earOptions ){				
				ant.fileset	( dir: enterpriseFolder )
				ant.fileset ( dir: librariesFolder )
		}
		
		ant.property( name : "global.artifact.location", value : earDest )
	}
	
	/**
	 * Method to remove version number from file name
	 * @param location Folder containing the files to be renamed
	 */
	def renameFiles(String location) {
		def dir = new File(location)
		if(dir.exists() && dir.isDirectory()) {
			dir.eachFile() { file ->
				if(!file.isDirectory()) {
					def name = file.getName()
					def newName, newExtension
					def requiredIndex = -1
					def lastIndex =  name.lastIndexOf('.')
					name.eachWithIndex() { aChar, index->
						if(aChar.equals('-')) {
							boolean isDigit = Character.isDigit(name.charAt(index+1))
							if(isDigit)
								requiredIndex = index
						}
					}
					if(requiredIndex != -1)
						newName = name.substring(0,requiredIndex)
					if(lastIndex != -1)
						newExtension = name.substring(lastIndex)
					if(newName!= null && newExtension!= null) {
						File newFile = new File(location+newName+newExtension)
						file.renameTo(newFile)
					}
				}
			}	
		}
	}
	
}